@@ -2,6 +2,7 @@
 from loguru import logger
 from uuid import uuid4
 from openai import AsyncOpenAI
+from openai.types.responses import ResponseFunctionToolCall, ResponseOutputMessage, ResponseOutputText
 
 from AgentService.db import Db
 from AgentService.dtypes.db import method as dmth
@@ -73,24 +74,53 @@ class Agent:
             tool_choice="auto"
         )
 
-        message = response.choices[0].message
-        if message.tool_calls:
+        response_types = []
+        for i in response.output:
+            if i.__class__ in response_types:
+                continue
+
+            response_types.append(i.__class__)
+
+        if len(response_types) != 1:
+            raise TypeError(f"Got many response types in 1 output -> {response_types}")
+
+        if isinstance(response_types[0], ResponseFunctionToolCall):
             return AgentResponse(
                 type=AgentResponseType.tools,
                 content=[
                     Tool(
-                        id=tool_call.id,
-                        name=tool_call.function.name,
-                        arguments=tool_call.function.arguments
+                        id=i.call_id,
+                        name=i.name,
+                        arguments=i.arguments
                     )
-                    for tool_call in message.tool_calls
+                    for i in response.output
                 ]
             )
 
-        return AgentResponse(
-            type=AgentResponseType.answer,
-            content=message.content
-        )
+        elif isinstance(response_types[0], ResponseOutputMessage):
+            content_types = []
+            for i in response.output:
+                if i.content.__class__ in content_types:
+                    continue
+
+                content_types.append(i.content.__class__)
+
+            if len(content_types) != 1:
+                self.log.warning(f"Got many content types in 1 output -> {content_types}")
+
+            if not isinstance(content_types[0], ResponseOutputText):
+                self.log.warning(f"Unsupported content type -> {content_types[0]}")
+
+            return AgentResponse(
+                type=AgentResponseType.answer,
+                content="\n".join([
+                    i.content.text
+                    for i in response.output if isinstance(i.content, ResponseOutputText)
+                ])
+            )
+
+        else:
+            raise TypeError(f"Unsupported response type -> {response_types[0]}")
 
     async def generate(self, chat: Chat) -> AgentResponse:
         try:
